package org.infinispan.wfink.clustered;

/*
 * JBoss, Home of Professional Open Source
 * Copyright 2014, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Map;
import java.util.logging.Logger;

import org.infinispan.Cache;
import org.infinispan.CacheStream;
import org.infinispan.commons.util.CloseableIterator;
import org.infinispan.configuration.cache.CacheMode;
import org.infinispan.configuration.cache.ConfigurationBuilder;
import org.infinispan.configuration.global.GlobalConfigurationBuilder;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.partitionhandling.PartitionHandling;

/**
 * @author Wolf-Dieter Fink
 */
public class ClusteredIterationStandalone {
   private final Logger log = Logger.getLogger("TEST");
   private final DefaultCacheManager cacheManager;
   private Cache<String, String> cache;

	public ClusteredIterationStandalone() {

		// Setup up a clustered cache manager
		GlobalConfigurationBuilder global = GlobalConfigurationBuilder.defaultClusteredBuilder();
		// Initialize the cache manager
		cacheManager = new DefaultCacheManager(global.build());
		// Create cache configuration
		ConfigurationBuilder builder = new ConfigurationBuilder();
		builder.clustering().cacheMode(CacheMode.DIST_ASYNC)
		.partitionHandling().whenSplit(PartitionHandling.DENY_READ_WRITES)  // this configuration will cause reactive to fail other PH ALLOW_READS or ALLOW_READ_WRITES work as expected
		.hash()
		.numOwners(4);
		
		// Obtain a cache
		
		cacheManager.defineConfiguration("testCache", builder.build());
		this.cache = cacheManager.getCache("testCache");
	}


  public void stop() {
      cacheManager.stop();
   }

	private void loop1() {
		try (CacheStream<String> cacheStream = cache.values().stream()) {
			try {
				cacheStream.forEach(it -> {
					// logic
				});
				log.info("done stream.foreach");
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
	}

	private void loop2() {
		try (CloseableIterator<Map.Entry<String,String>> iterator = cache.entrySet().iterator()) {
			try {
				while (iterator.hasNext()) {
					Object key = iterator.next();
				}
				log.info("done iterator");
			} catch (Throwable t) {
				log.severe("error iterator " + t);
				t.printStackTrace();
			}
		}
	}

	private void loop3() {
		try (CacheStream<String> cacheStream = cache.keySet().stream().distributedBatchSize(cache.size())) {
			try {
				cacheStream.forEach(it -> {
					// logic
				});
				log.info("done stream( distributedBatchSize). foreach");
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
	}

	private void loop4() {

		try (CacheStream<String> cacheStream = cache.keySet().stream().distributedBatchSize(cache.size() - 2)) {
			try {
				cacheStream.forEach(it -> {
				});
				log.info("done stream(distributedBatchSize - 2). foreach");
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
	}
			   
   private void populate() {
	   for (int i = 0; i < 1000000; i++) {
		   cache.put(String.valueOf(i), String.valueOf(i));
	   }
	   log.info(" ----------------    cache size " + cache.size() + "   ------------------------");
   }

   public static void main(String[] args) {
      ClusteredIterationStandalone main = new ClusteredIterationStandalone();
      
      main.populate();

      main.loop1();
      main.loop2();
      main.loop3();
      main.loop4();

      main.stop();
   }
}
